const dgram = require('dgram');
const radius = require('radius');

const RADIUS_PORT = 1812; // Default RADIUS authentication port
const RADIUS_SECRET = 'secret';

const server = dgram.createSocket('udp4');

server.on('message', (msg, rinfo) => {
    try {
        const packet = radius.decode({ packet: msg, secret: RADIUS_SECRET });

        // Log the client's IP address
        console.log(`Received RADIUS packet from ${rinfo.address}:${rinfo.port}`);

        if (packet.code !== 'Access-Request') {
            console.log(`Unknown packet type: ${packet.code}`);
            return;
        }

        const username = packet.attributes['User-Name'];

        // Decrypt the password
        const encryptedPassword = packet.attributes['User-Password'];
        const password = radius.decrypt({ packet: packet, secret: RADIUS_SECRET, attribute: encryptedPassword });

        console.log(`Username: ${username}, Password: ${password}`);

        // Implement your authentication logic here
        const isAuthenticated = authenticateUser(username, password);

        const response = isAuthenticated ? 'Access-Accept' : 'Access-Reject';
        const responsePacket = radius.encode_response({
            packet: packet,
            code: response,
            secret: RADIUS_SECRET,
        });

        server.send(responsePacket, 0, responsePacket.length, rinfo.port, rinfo.address, (err, bytes) => {
            if (err) {
                console.error('Error sending response:', err);
            }
        });
    } catch (err) {
        console.error('Error decoding RADIUS packet:', err);
    }
});

server.on('listening', () => {
    const address = server.address();
    console.log(`RADIUS server listening on ${address.address}:${address.port}`);
});

server.bind(RADIUS_PORT);

function authenticateUser(username, password) {
    // Replace this with your authentication logic (e.g., checking against a database)
    return username === 'ali' && password === 'ahmad';
}
